;PS/2 Interface by Roman Vaughan
;
;http://www.davevaughan.com/roman/
;
;Created on 20-11-2009

;-----------------------------------------------------
;Returns the state of the Clock and Data
;-----------------------------------------------------
;
;{
PS2_read_CLOCK
	btfss	PS2_CLKi
	retlw	0x00
	retlw	0x01

PS2_read_DATA
	btfss	PS2_DATAi
	retlw	0x00
	retlw	0x01
;}	

;-----------------------------------------------------
;Sets the Data or Clock signal active 'low' or float 'high'
;-----------------------------------------------------
;
;{
PS2_set_CLOCK
	movwf	PS2_pSet
	btfsc	PS2_pSet,0
		goto	set_clk
	bsf		STATUS,RP0
		bcf	PS2_TRIS_CLK
	bcf		STATUS,RP0
	bcf		PS2_CLKi
	return
set_clk
	bsf		PS2_CLKi
	bsf		STATUS,RP0
		bsf	PS2_TRIS_CLK
	bcf		STATUS,RP0
	return

PS2_set_DATA
	movwf	PS2_pSet
	btfsc	PS2_pSet,0
		goto	set_data
	bsf		STATUS,RP0
		bcf	PS2_TRIS_DATA
	bcf		STATUS,RP0
	bcf		PS2_DATAi
	return
set_data
	bsf		PS2_DATAi
	bsf		STATUS,RP0
		bsf	PS2_TRIS_DATA
	bcf		STATUS,RP0
	return
;}
;-----------------------------------------------------
;Waits for the clock or data signal to change to the desired state
;-----------------------------------------------------
;
;{
PS2_wait_CLOCK
	movwf	PS2_state
swait
	call	PS2_read_CLOCK
	xorwf	PS2_state,w
	btfss	STATUS,Z
	goto	swait
	retlw 0x00
	
PS2_wait_Data
	movwf	PS2_state
dwait
	call	PS2_read_DATA
	xorwf	PS2_state,w
	btfss	STATUS,Z
	goto	dwait
	retlw 0x00
;}	
;-----------------------------------------------------
;Reads the incomming data from the PS/2 Device
;-----------------------------------------------------
;
;{
PS2_get
	clrf	PS2_pariaty
	movlw	0x01			;
	call	PS2_set_CLOCK	;Bring Clock 'High'
	
	movlw	0x00			;
	call	PS2_wait_CLOCK	;Wait for Clock to go 'Low'
	
	movlw	0x01			;
	call	PS2_wait_CLOCK	;Wait for clock to go 'High'
	
	movlw	0x08			;
	movwf	PS2_COUNT		;Store 8 into counter
get_l1;start of loop1
	movlw	0x00			;
	call	PS2_wait_CLOCK	;Wait for clock to go 'Low'
	
	
	rrf		PS2_temp,F		;rotate the temp byte right by 1
	call	PS2_read_DATA	;read incomming DATA
	movwf	PS2_temp2		;store it into temp2 for testing
	addwf	PS2_pariaty,F	;add it to the parity bit checker
	btfsc	PS2_temp2,0		;test the bit jsut read
	goto	$+3
	bcf		PS2_temp,7		;clear the 8th bit
	goto	$+2
	bsf		PS2_temp,7		;set the 8th bit
	
	movlw	0x01			;
	call	PS2_wait_CLOCK	;Wait for Clock to go 'High
	
	
	decfsz	PS2_COUNT,F		;decrement counter by 1 and skip next instruction if Zero
	goto	get_l1				;
get_el1;end of loop 1
	movlw	0x00			;
	call	PS2_wait_CLOCK	;Wait for Clock to go 'Low'
	
	call	PS2_read_DATA	;Read Parity bit
	addwf	PS2_pariaty,F	;add it to pariaty bit checker
	btfsc	PS2_pariaty,0	;check to see if we have an odd amount of parity bits
	goto$+3
		movlw	0x03		;call pariaty bit error
		call	PS2_error	;Error, there is an even amount of Parity bits
	movlw	0x01			;
	call	PS2_wait_CLOCK	;Wait for Clock to go 'Low'
	
	movlw	0x00			;
	call	PS2_wait_CLOCK	;Wait for Clock to go 'High'
	
	call	PS2_read_DATA	;read the final stop bit
	movwf	PS2_temp2		;store the stop bit into temp2
	btfsc	PS2_temp2,0		;check to see if the stop bit is valid
	goto	$+3
		movlw	0x04		
		call	PS2_error	;Error, the stop bit is invalid, should be 'high' not 'low'
	
	movlw	0x00			;
	call	PS2_set_CLOCK	;set the clock low preventing the PS/2 device from sending more data
	
	movlw	d'50'			;
	call	DelayUS			;give it a 50us delay
	
	movfw	PS2_temp		;Move the temp byte into the wreg
	return					;and return
;}
;-----------------------------------------------------
;Writes data to the PS/2 Device
;-----------------------------------------------------
;
;{	
PS2_send
	movwf	PS2_data
	movwf	PS2_temp
	movlw	0x00
	call	PS2_set_CLOCK
	movlw	d'100'
	call	DelayUS
	movlw	0x00
	call	PS2_set_DATA
	
	movlw	0x01
	call	PS2_set_CLOCK
	
	clrf	PS2_pariaty
	
	movlw	0x08			;
	movwf	PS2_COUNT		;Store 8 into counter
set_l1;start of loop1
	movlw	0x00			;
	call	PS2_wait_CLOCK		;Wait for clock to go 'Low'
	clrw
	movfw	PS2_temp
	andlw	0x01
	call	PS2_set_DATA
	
	movfw	PS2_temp
	andlw	0x01
	addwf	PS2_pariaty,F
	rrf		PS2_temp,F		;rotate the temp byte right by 1
	
	movlw	0x01			;
	call	PS2_wait_CLOCK	;Wait for Clock to go 'High
	
	decfsz	PS2_COUNT,F		;decrement counter by 1
	goto	set_l1
set_el1;end of loop 1
	movlw	0x00
	call	PS2_wait_CLOCK
	movfw	PS2_pariaty		;\
	addlw	0x01			;|This sends out a Odd Parity bit
	;movwf	PS2_pariaty		;|
	call	PS2_set_DATA	;/
	
	movlw	0x01
	call	PS2_wait_CLOCK
	movlw	0x00
	call	PS2_wait_CLOCK
	movlw	0x01
	call	PS2_set_DATA
	movlw	0x01
	call	PS2_wait_CLOCK
	movlw	0x00
	call	PS2_wait_CLOCK
	call	PS2_read_DATA
	movwf	PS2_temp
	btfss	PS2_temp,0		;test for line control, 'high' means error!
	goto	$+3
		movlw	d'8'
		goto	PS2_error
	;movlw	0x00
	;call	PS2_wait_Data
	movlw	0x01
	call	PS2_wait_CLOCK
	call	PS2_get
	movwf	PS2_acknoledge
	xorlw	0xFA
	btfsc	STATUS,Z
	retlw	0x00			;return after successful ACK
	movfw	PS2_acknoledge
	xorlw	0xFE			;test for "re-send"
	goto	$+3				;yes its a "re-send", so resened that last byte
		movlw	0x05		;nope we have a error most likely 0xFC
		call	PS2_error	;
	movfw	PS2_data
	goto	PS2_send
;}
;-----------------------------------------------------
;Special function for Touchpad devices
;-----------------------------------------------------
;
;{
PS2_send_tp_arg	;sends b'10110100' as b'00000010',b'00000011',b'00000001' and b'00000000';
	movwf	PS2_tp_arg
	movwf	PS2_tp_arg2
	movlw	0xE8
	call	PS2_send
	
	movlw	0x06
	movwf	PS2_COUNTB
tp_l1
	rrf		PS2_tp_arg2,F
	decfsz	PS2_COUNTB,F
	goto	tp_l1
	movfw	PS2_tp_arg2
	andlw	0x03
	call	PS2_send
	movfw	PS2_tp_arg
	movwf	PS2_tp_arg2
	
	movlw	0xE8
	call	PS2_send
	movlw	0x04
	movwf	PS2_COUNTB
tp_l2
	rrf		PS2_tp_arg2,F
	decfsz	PS2_COUNTB,F
	goto	tp_l2
	movfw	PS2_tp_arg2
	andlw	0x03
	call	PS2_send
	movfw	PS2_tp_arg
	movwf	PS2_tp_arg2
	
	movlw	0xE8
	call	PS2_send
	movlw	0x02
	movwf	PS2_COUNTB
tp_l3
	rrf		PS2_tp_arg2,F
	decfsz	PS2_COUNTB,F
	goto	tp_l3
	movfw	PS2_tp_arg2
	andlw	0x03
	call	PS2_send
	movfw	PS2_tp_arg
	movwf	PS2_tp_arg2
	
	movlw	0xE8
	call	PS2_send
	movfw	PS2_tp_arg2
	andlw	0x03
	call	PS2_send
	return
;}	

PS2_init
	movlw	0xFF			;
	call	PS2_send		;send RESET Command
PS2_init_wait
	call	PS2_get			;\
	xorlw	0xAA			;|
	btfsc	STATUS,Z		;|
	goto	$+3				; \
		goto	PS2_init_wait;|Checks to make sure the device starts up properly
	call	PS2_get			; /
	xorlw	0x00			;|
	btfsc	STATUS,Z		;|
	goto	$+3				;|
		movlw	0x07		;|
		call	PS2_error	;/
		
	movlw	0x00			;\		Special to synaptic touchpads only
	call	PS2_send_tp_arg	;|	sends a unique (to touchpads) command set that requests	the device's model number
	movlw	0xE9			;|	Sending 0xE9 after the special command will return a 3 byte data with the model-
	call	PS2_send		;|   number, device type and rev
	call	PS2_get			;|  If we get 0x00 then its a normal mouse
	movwf	PS2_minor		;| 
	call	PS2_get			;|
	movwf	PS2_type		;/ - Device type (0x00 if its a normal PS/2 mouse)
	
	xorlw	0x47			;\
	btfsc	STATUS,Z		;|
	goto	$+3				;|Checks to see if we have a touch pad
		movlw	0x06		;|
		call	PS2_error	;/
	call	PS2_get
	andlw	0x0F
	movwf	PS2_major
	
	movlw	0xF0			;sets the device to remote mode meaning only send data when requested by 0xEB
	call	PS2_send		;
	movlw	0x80			; Sends a 'set modes' to the touchpad
	call	PS2_send_tp_arg	;
	movlw	0xF3			;\
	call	PS2_send		;|set sample rate
	movlw	0x14			;|sample rate argument d'20'
	call	PS2_send		;/
	movlw	0xF4			; enable command
	call	PS2_send		;
	
	movlw	0x01			;bring clock back up high for device to transmit data when ready
	call	PS2_set_CLOCK	;
	retlw	0x00
	
PS2_update
	movlw	0xEB
	call	PS2_send
	
	call	PS2_get
	movwf	PS2_button
	
	call	PS2_get
	movwf	PS2_XY
	
	call	PS2_get
	movwf	PS2_pressure
	
	call	PS2_get
	movwf	PS2_XY2
	
	call	PS2_get
	movwf	PS2_X
	
	call	PS2_get
	movwf	PS2_Y
	
	retlw	0x00